@@ -251,7 +251,8 @@ Alias for $req->body.
 =head2 $req->match
 
 This contains the matching part of a Regex action. Otherwise
-it returns the same as 'action'.
+it returns the same as 'action', except for default actions,
+which return an empty string.
 
 =head2 $req->method
 
@@ -365,7 +366,6 @@ sub path {
     my $path     = $self->uri->path;
     my $location = $self->base->path;
     $path =~ s/^(\Q$location\E)?//;
-    $path =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/eg;
     $path =~ s/^\///;
     $self->{path} = $path;
 
